package cat.hack3.codingtests.marsrover.ui;

import cat.hack3.codingtests.marsrover.Coordinates;
import cat.hack3.codingtests.marsrover.Direction;
import cat.hack3.codingtests.marsrover.MarsMap;
import cat.hack3.codingtests.marsrover.MarsRover;

import java.util.InputMismatchException;
import java.util.List;
import java.util.Scanner;

import static cat.hack3.codingtests.marsrover.ui.UICommons.output;

public class RoverInitializer {
    private final Scanner reader;
    private final DirectionRetriever directionRetriever;


    public RoverInitializer(Scanner reader) {
        this.reader = reader;
        directionRetriever = new DirectionRetriever(reader);
    }

    MarsRover initializeFromUserInputs() {
        output("Please, introduce the latitude  map size:");
        int mapHeight = takeIntegerInput();
        output("Please, introduce the longitude map size:");
        int mapWidth = takeIntegerInput();

        output("Great! Now please introduce the initial location of the Rover.");
        output("Latitude:");
        int latitudeStartingPoint = takeIntegerInput();
        output("Longitude:");
        int longitudeStartingPoint = takeIntegerInput();

        Direction startingDirection = directionRetriever.retrieveDirection();

        MarsRover rover = deployRover(mapHeight, mapWidth, latitudeStartingPoint, longitudeStartingPoint, startingDirection);

        output(PresentationMessage.READY_MESSAGE);
        reader.next(); //input ignored

        return rover;
    }

    private int takeIntegerInput() {
        try {
            return reader.nextInt();
        } catch (InputMismatchException e) {
            output("(please, introduce a number)");
            reader.next();
            return takeIntegerInput();
        }
    }

    MarsRover autoInitialize() { //to test more quickly
        List<Coordinates> obstacles = List.of(
                Coordinates.of(3, 3),
                Coordinates.of(5, 5),
                Coordinates.of(7, 7),
                Coordinates.of(9, 9),
                Coordinates.of(10, 10)
        );
        return deployRover(10, 10, 2, 3, Direction.SOUTH, obstacles);
    }

    @SafeVarargs
    private MarsRover deployRover(int mapHeight, int mapWidth, int latitudeStartingPoint, int longitudeStartingPoint, Direction startingDirection, List<Coordinates>... obstaclesLocalizations) {
        var startingCoordinates = Coordinates.of(latitudeStartingPoint, longitudeStartingPoint);
        var marsMap = new MarsMap(mapHeight, mapWidth, startingCoordinates, obstaclesLocalizations);
        return new MarsRover(marsMap, startingDirection);
    }
}
